@@ -1,7 +1,7 @@
 // samplv1_controls.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -234,7 +234,7 @@ public:
 			m_mask = new_size - 1;
 			m_events = new_events;
 			if (old_events)
-				delete old_events;
+				delete [] old_events;
 		}
 	}
 
