@@ -1,7 +1,7 @@
 // samplv1_ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -126,4 +126,22 @@ void samplv1_ui::updatePreset ( bool bDirty )
 }
 
 
+void samplv1_ui::midiInEnabled ( bool bEnabled )
+{
+	m_pSampl->midiInEnabled(bEnabled);
+}
+
+
+uint32_t samplv1_ui::midiInCount (void)
+{
+	return m_pSampl->midiInCount();
+}
+
+
+void samplv1_ui::directNoteOn ( int note, int vel )
+{
+	m_pSampl->directNoteOn(note, vel);
+}
+
+
 // end of samplv1_ui.cpp
