@@ -1,7 +1,7 @@
 // samplv1_ui.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -62,6 +62,11 @@ public:
 
 	void updatePreset(bool bDirty);
 
+	void midiInEnabled(bool bEnabled);
+	uint32_t midiInCount();
+
+	void directNoteOn(int note, int vel);
+
 private:
 
 	samplv1 *m_pSampl;
