@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,6 +30,10 @@
 
 #include <QMessageBox>
 #include <QDir>
+#include <QTimer>
+
+#include <QShowEvent>
+#include <QHideEvent>
 
 
 //-------------------------------------------------------------------------
@@ -138,7 +142,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QStringList states;
 	states << tr("Off");
 	states << tr("On");
-
+#if 0
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
 	m_ui.Gen1LoopKnob->insertItems(0, states);
 
@@ -146,7 +150,9 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
-
+#else
+	m_ui.Lfo1SyncKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+#endif
 	// Special values
 	const QString& sOff = states.first();
 	m_ui.Gen1GlideKnob->setSpecialValueText(sOff);
@@ -184,7 +190,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Lfo1BpmKnob->setScale(1.0f);
 	m_ui.Lfo1BpmKnob->setMinimum(0.0f);
 	m_ui.Lfo1BpmKnob->setMaximum(360.0f);
-	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
+//	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
 	m_ui.Lfo1SweepKnob->setMinimum(-1.0f);
 	m_ui.Lfo1SweepKnob->setMaximum(+1.0f);
 	m_ui.Lfo1CutoffKnob->setMinimum(-1.0f);
@@ -205,10 +211,12 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		channels << QString::number(iChannel + 1);
 
 	m_ui.Def1ChannelKnob->insertItems(0, channels);
-
+#if 0
 	// Mono switches
 	m_ui.Def1MonoKnob->insertItems(0, states);
-
+#else
+	m_ui.Def1MonoKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+#endif
 	// Output (stereo-)width limits.
 	m_ui.Out1WidthKnob->setMinimum(-1.0f);
 	m_ui.Out1WidthKnob->setMaximum(+1.0f);
@@ -221,7 +229,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setScale(1.0f);
 	m_ui.Del1BpmKnob->setMinimum(0.0f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
-	m_ui.Del1BpmKnob->setSingleStep(1.0f);
+//	m_ui.Del1BpmKnob->setSingleStep(1.0f);
 
 	// Reverb (stereo-)width limits.
 	m_ui.Rev1WidthKnob->setMinimum(-1.0f);
@@ -504,13 +512,15 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	if (pConfig) {
 		samplv1widget_dial::setDialMode(
 			samplv1widget_dial::DialMode(pConfig->iKnobDialMode));
+		samplv1widget_edit::setEditMode(
+			samplv1widget_edit::EditMode(pConfig->iKnobEditMode));
 	}
 
 	// Epilog.
 	// QWidget::adjustSize();
 
 	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
-	m_ui.StatusBar->setModified(false);
+	m_ui.StatusBar->modified(false);
 	m_ui.Preset->setDirtyPreset(false);
 }
 
@@ -523,13 +533,11 @@ samplv1widget::~samplv1widget (void)
 }
 
 
-// Create/initialize the scheduler/work notifier.
-void samplv1widget::initSchedNotifier (void)
+// Open/close the scheduler/work notifier.
+void samplv1widget::openSchedNotifier (void)
 {
-	if (m_sched_notifier) {
-		delete m_sched_notifier;
-		m_sched_notifier = NULL;
-	}
+	if (m_sched_notifier)
+		return;
 
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi == NULL)
@@ -540,29 +548,61 @@ void samplv1widget::initSchedNotifier (void)
 	QObject::connect(m_sched_notifier,
 		SIGNAL(notify(int, int)),
 		SLOT(updateSchedNotify(int, int)));
+
+	pSamplUi->midiInEnabled(true);
+}
+
+
+void samplv1widget::closeSchedNotifier (void)
+{
+	if (m_sched_notifier) {
+		delete m_sched_notifier;
+		m_sched_notifier = NULL;
+	}
+
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi)
+		pSamplUi->midiInEnabled(false);
+}
+
+
+// Show/hide widget handlers.
+void samplv1widget::showEvent ( QShowEvent *pShowEvent )
+{
+	QWidget::showEvent(pShowEvent);
+
+	openSchedNotifier();
+}
+
+
+void samplv1widget::hideEvent ( QHideEvent *pHideEvent )
+{
+	closeSchedNotifier();
+
+	QWidget::hideEvent(pHideEvent);
 }
 
 
 // Param kbob (widget) map accesors.
-void samplv1widget::setParamKnob ( samplv1::ParamIndex index, samplv1widget_knob *pKnob )
+void samplv1widget::setParamKnob ( samplv1::ParamIndex index, samplv1widget_param *pParam )
 {
-	pKnob->setDefaultValue(samplv1_param::paramDefaultValue(index));
+	pParam->setDefaultValue(samplv1_param::paramDefaultValue(index));
 
-	m_paramKnobs.insert(index, pKnob);
-	m_knobParams.insert(pKnob, index);
+	m_paramKnobs.insert(index, pParam);
+	m_knobParams.insert(pParam, index);
 
-	QObject::connect(pKnob,
+	QObject::connect(pParam,
 		SIGNAL(valueChanged(float)),
 		SLOT(paramChanged(float)));
 
-	pKnob->setContextMenuPolicy(Qt::CustomContextMenu);
+	pParam->setContextMenuPolicy(Qt::CustomContextMenu);
 
-	QObject::connect(pKnob,
+	QObject::connect(pParam,
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(paramContextMenu(const QPoint&)));
 }
 
-samplv1widget_knob *samplv1widget::paramKnob ( samplv1::ParamIndex index ) const
+samplv1widget_param *samplv1widget::paramKnob ( samplv1::ParamIndex index ) const
 {
 	return m_paramKnobs.value(index, NULL);
 }
@@ -574,9 +614,9 @@ void samplv1widget::setParamValue (
 {
 	++m_iUpdate;
 
-	samplv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob)
-		pKnob->setValue(fValue, bDefault);
+	samplv1widget_param *pParam = paramKnob(index);
+	if (pParam)
+		pParam->setValue(fValue, bDefault);
 
 	updateParamEx(index, fValue);
 
@@ -587,9 +627,9 @@ float samplv1widget::paramValue ( samplv1::ParamIndex index ) const
 {
 	float fValue = 0.0f;
 
-	samplv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob) {
-		fValue = pKnob->value();
+	samplv1widget_param *pParam = paramKnob(index);
+	if (pParam) {
+		fValue = pParam->value();
 	} else {
 		samplv1_ui *pSamplUi = ui_instance();
 		if (pSamplUi)
@@ -606,14 +646,14 @@ void samplv1widget::paramChanged ( float fValue )
 	if (m_iUpdate > 0)
 		return;
 
-	samplv1widget_knob *pKnob = qobject_cast<samplv1widget_knob *> (sender());
-	if (pKnob) {
-		const samplv1::ParamIndex index = m_knobParams.value(pKnob);
+	samplv1widget_param *pParam = qobject_cast<samplv1widget_param *> (sender());
+	if (pParam) {
+		const samplv1::ParamIndex index = m_knobParams.value(pParam);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
-			.arg(pKnob->toolTip())
-			.arg(pKnob->valueText()), 5000);
+			.arg(pParam->toolTip())
+			.arg(pParam->valueText()), 5000);
 		updateDirtyPreset(true);
 	}
 }
@@ -659,14 +699,14 @@ void samplv1widget::updateSchedParam ( samplv1::ParamIndex index, float fValue )
 {
 	++m_iUpdate;
 
-	samplv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob) {
-		pKnob->setValue(fValue, false);
+	samplv1widget_param *pParam = paramKnob(index);
+	if (pParam) {
+		pParam->setValue(fValue, false);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
-			.arg(pKnob->toolTip())
-			.arg(pKnob->valueText()), 5000);
+			.arg(pParam->toolTip())
+			.arg(pParam->valueText()), 5000);
 		updateDirtyPreset(true);
 	}
 
@@ -688,11 +728,12 @@ void samplv1widget::resetParams (void)
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		float fValue = samplv1_param::paramDefaultValue(index);
-		samplv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob)
-			fValue = pKnob->defaultValue();
+		samplv1widget_param *pParam = paramKnob(index);
+		if (pParam && pParam->isDefaultValue())
+			fValue = pParam->defaultValue();
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
+	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
 	}
 
@@ -714,9 +755,9 @@ void samplv1widget::swapParams ( bool bOn )
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		samplv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob) {
-			const float fOldValue = pKnob->value();
+		samplv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			const float fOldValue = pParam->value();
 			const float fNewValue = m_params_ab[i];
 			setParamValue(index, fNewValue);
 			updateParam(index, fNewValue);
@@ -779,9 +820,9 @@ void samplv1widget::resetParamValues (void)
 void samplv1widget::resetParamKnobs (void)
 {
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		samplv1widget_knob *pKnob = paramKnob(samplv1::ParamIndex(i));
-		if (pKnob)
-			pKnob->resetDefaultValue();
+		samplv1widget_param *pParam = paramKnob(samplv1::ParamIndex(i));
+		if (pParam)
+			pParam->resetDefaultValue();
 	}
 }
 
@@ -917,6 +958,10 @@ void samplv1widget::loadSampleFile ( const QString& sFilename )
 // Sample updater (crude experimental stuff II).
 void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 {
+	samplv1_ui *pSamplUi = ui_instance();
+	if (m_ui.Gen1Sample->instance() == NULL)
+		m_ui.Gen1Sample->setInstance(pSamplUi);
+
 	m_ui.Gen1Sample->setSample(pSample);
 
 	++m_iUpdate;
@@ -943,6 +988,17 @@ void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 }
 
 
+// Sample playback (direct note-on/off).
+void samplv1widget::playSample (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("samplv1widget::playSample()");
+#endif
+
+	m_ui.Gen1Sample->directNoteOn();
+}
+
+
 // MIDI note/octave name helper (static).
 QString samplv1widget::noteName ( int note )
 {
@@ -1093,6 +1149,10 @@ void samplv1widget::contextMenuRequest ( const QPoint& pos )
 		QIcon(":/images/fileOpen.png"),
 		tr("Open Sample..."), this, SLOT(openSample()));
 	pAction->setEnabled(pSamplUi != NULL);
+	pAction = menu.addAction(
+		QIcon(":/images/playSample.png"),
+		tr("Play"), this, SLOT(playSample()));
+	pAction->setEnabled(pSamplUi != NULL);
 	menu.addSeparator();
 	pAction = menu.addAction(
 		tr("Reset"), this, SLOT(clearSample()));
@@ -1125,10 +1185,16 @@ void samplv1widget::updateSchedNotify ( int stype, int sid )
 		return;
 
 #ifdef CONFIG_DEBUG
-	qDebug("samplv1widget::updateSchedNotify(%d, %d)", stype, sid);
+	qDebug("samplv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (samplv1_sched::Type(stype)) {
+	case samplv1_sched::MidiIn:
+		if (pSamplUi->midiInCount() > 0) {
+			m_ui.StatusBar->midiInLed(true);
+			QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
+		}
+		break;
 	case samplv1_sched::Controller: {
 		samplv1widget_control *pInstance
 			= samplv1widget_control::getInstance();
@@ -1162,6 +1228,13 @@ void samplv1widget::updateSchedNotify ( int stype, int sid )
 }
 
 
+// MIDI In LED timeout.
+void samplv1widget::midiInLedTimeout (void)
+{
+	m_ui.StatusBar->midiInLed(false);
+}
+
+
 // Menu actions.
 void samplv1widget::helpConfigure (void)
 {
@@ -1206,12 +1279,11 @@ void samplv1widget::helpAbout (void)
 	sText += "<b>" SAMPLV1_TITLE "</b> - " + tr(SAMPLV1_SUBTITLE) + "<br />\n";
 	sText += "<br />\n";
 	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
-	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
-	QStringListIterator iter(list);
-	while (iter.hasNext()) {
+//	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
+	if (!list.isEmpty()) {
 		sText += "<small><font color=\"red\">";
-		sText += iter.next();
-		sText += "</font></small><br />";
+		sText += list.join("<br />\n");
+		sText += "</font></small><br />\n";
 	}
 	sText += "<br />\n";
 	sText += tr("Website") + ": <a href=\"" SAMPLV1_WEBSITE "\">" SAMPLV1_WEBSITE "</a><br />\n";
@@ -1242,7 +1314,7 @@ void samplv1widget::updateDirtyPreset ( bool bDirtyPreset )
 	if (pSamplUi)
 		pSamplUi->updatePreset(bDirtyPreset);
 
-	m_ui.StatusBar->setModified(bDirtyPreset);
+	m_ui.StatusBar->modified(bDirtyPreset);
 	m_ui.Preset->setDirtyPreset(bDirtyPreset);
 }
 
@@ -1250,9 +1322,9 @@ void samplv1widget::updateDirtyPreset ( bool bDirtyPreset )
 // Param knob context menu.
 void samplv1widget::paramContextMenu ( const QPoint& pos )
 {
-	samplv1widget_knob *pKnob
-		= qobject_cast<samplv1widget_knob *> (sender());
-	if (pKnob == NULL)
+	samplv1widget_param *pParam
+		= qobject_cast<samplv1widget_param *> (sender());
+	if (pParam == NULL)
 		return;
 
 	samplv1_ui *pSamplUi = ui_instance();
@@ -1272,9 +1344,9 @@ void samplv1widget::paramContextMenu ( const QPoint& pos )
 		QIcon(":/images/samplv1_control.png"),
 		tr("MIDI &Controller..."));
 
-	if (menu.exec(pKnob->mapToGlobal(pos)) == pAction) {
-		const samplv1::ParamIndex index = m_knobParams.value(pKnob);
-		const QString& sTitle = pKnob->toolTip();
+	if (menu.exec(pParam->mapToGlobal(pos)) == pAction) {
+		const samplv1::ParamIndex index = m_knobParams.value(pParam);
+		const QString& sTitle = pParam->toolTip();
 		samplv1widget_control::showInstance(pControls, index, sTitle, this);
 	}
 }
