@@ -1,7 +1,7 @@
 // samplv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -73,7 +73,7 @@ samplv1widget_jack::samplv1widget_jack ( samplv1_jack *pSampl )
 	m_pSamplUi = new samplv1_ui(m_pSampl);
 
 	// May initialize the scheduler/work notifier.
-	initSchedNotifier();
+	openSchedNotifier();
 
 	// Initialize preset stuff...
 	// initPreset();
